#!/usr/bin/python
#-*- coding: utf-8 -*-

import cPickle as pickle, hashlib, re, sys
import Arguments, Errors

from ..__init__ import tokenize, prepare

class Text(object):
    def __init__(self, text, tags, about):
        if type(text) != str:
            raise TypeError('"text" argument must be a str, not %s.' %(type(text)))
        if type(tags) != list:
            raise TypeError('"tags" argument must be a list, not %s.' %(type(tags)))
        if type(about) != dict:
            raise TypeError('"about" argument must be a dict, not %s.' %(type(about)))
        for i in tags:
            if type(i) != Tag:
                raise TypeError('"tags" list has a unacceptable element type. "%s" <%s>' %(str(i), type(i)))
        
        self.text = text
        self.tags = tags
        self.about = about
        self._id_ = hashlib.md5(text).hexdigest().upper().upper()
    
        self.__edit__()
    
    def __edit__(self):
        self.text = prepare(self.text)

class Token(object):
    def __init__(self, token):
        if type(token) != str:
            raise TypeError('"token" argument must be a str, not %s.' %(type(token)))
        
        self.token = token
        self._id_ = hashlib.md5(token).hexdigest().upper()
        self.adress = []
        self.bigrama = []
        self.bigramb = []
    
    def addAdress(self, text):
        if type(text) != Text:
            raise TypeError('"text" argument must be corpse.obj.Text, not %s.' %(type(text)))
        
        tokens = tokenize(text.text, gramize=False)
        try:
            count = tokens.count(self.token)
            c = 0
            adress = []
            if count > 1:
                for i in range(count):
                    adress.append(tokens.index(self.token, c))
                    c += 1
            elif count == 1:
                adress = adress.append(tokens.index(self.token))
        except ValueError:
            raise ValueError('"%s" not found in text<%s>' %(self.token, text._id_))
        adress = {text._id_ : adress}
        
        if not adress in self.adress:
            self.adress.append(adress)
        else:
            raise Errors.AlreadyFound('Text is already scanned for adressing.')
    
    def bigram(self, text, pass_ = True):
        if type(text) != Text:
            raise TypeError('"text" argument must be corpse.obj.Text, not %s.' %(type(text)))
        if type(pass_) != bool:
            raise TypeError('"pass_" argument must be bool, not %s.' %(type(pass_)))
        
        text = prepare(text.text)
        cents = tokenize(text)
        
        for i in cents:
            if i.count(self.token) > 1:
                count = 0
                for q in range(count):
                    self.bigrama.append(i[i.index(self.token, count)+1])
                    self.bigramb.append(i[i.index(self.token, count)-1])
                    count += 1
            elif i.count(self.token) == 1:
                self.bigrama.append(i[i.index(self.token)+1])
                self.bigramb.append(i[i.index(self.token)-1])
            else:
                if pass_ == True:
                    pass
                else:
                    raise Errors.NotFound('"%s" token not found in "%s" hashed text.' %(self.token, hashlib.md5(text)))
    
        
class Tag(object):
    def __init__(self, tag, about):
        if type(tag) != str:
            raise TypeError('"tag" argument must be a str, not %s.' %(type(tag)))
        if type(about) != dict:
            raise TypeError('"about" argument must be a dict, not %s.' %(type(about)))
        
        self.tag = tag
        self.about = about
        self.adress = []
        self._id_ = hashlib.md5(tag).hexdigest().upper()
    
    def addAdress(self, text):
        if type(text) != Text:
            raise TypeError('"text" argument must be corpse.obj.Text, not %s.' %(type(text)))
        
        for i in text.tags:
            if i.tag == self.tag and not i._id_ in self.adress:
                self.adress.append(i._id_)

class Database(object):
    def __init__(self, db):
        if type(db) != str:
            raise TypeError('"db" argument must be str, not %s.' %(type(db)))
        
        try:
            self.__cursor__ = open(db, 'rb+')
            self.__db__ = open(db, 'wb+')
        except IOError:
            open(db, 'w+')
            self.__cursor__ = open(db, 'rb+')
            self.__db__ = open(db, 'wb+')
            
        self.__data__ = {}
        self.__constructor__()
    
    def __constructor__(self):
        try:
            self.__data__['text'] = []
            self.__data__['token'] = []
            self.__data__['tag'] = []
        except EOFError:
            self.push()
            self.__data__['text'] = []
            self.__data__['token'] = []
            self.__data__['tag'] = []
    
    def push(self):
        pickle.dump(self.__data__, self.__db__)
        self.__constructor__()
    
    def close(self):
        self.__db__.close()
        self.__cursor__.close()
        
    def appendText(self, text, tokenizing = True, tagging = True, adressing = True):
        if type(text) != Text:
            raise TypeError('"text" argument must be corpse.obj.Text, not %s.' %(type(text)))
        
        notfound = False
        for i in self.__data__['text']:
            if i._id_ == text._id_:
                raise Errors.AlreadyFound('Specified text already found in database.')
            else:
                if notfound == False:
                    notfound = True
        if notfound == True:
            self.__data__['text'].append(text)
        
        if tokenizing == True:
            tokens = tokenize(prepare(text.text), gramize=False)
            found = []
            for i in tokens:
                if re.search('^"', i):
                    pass
                try:
                    self.appendToken(Token(i))
                except Errors.AlreadyFound:
                    found.append(i)
            
            if adressing == True:
                for i in found:
                    for q in self.__data__['token']:
                        if q._id_ == hashlib.md5(i).hexdigest().upper():
                            try:
                                q.addAdress(text)
                            except Errors.AlreadyFound:
                                pass
                        else:
                            pass
        
        if tagging == True:
            tags = text.tags
            found = []
            for i in tags:
                try:
                    self.appendTag(i)
                except Errors.AlreadyFound:
                    found.append(i)
            
            if adressing == True:
                for i in found:
                    for q in self.__data__['tag']:
                        if q._id_ == hashlib.md5(i.tag).hexdigest().upper():
                            try:
                                q.addAdress(text)
                            except Errors.AlreadyFound:
                                pass
                        else:
                            pass
    
    def appendToken(self, token):
        if type(token) != Token:
            raise TypeError('"token" argument must be corpse.obj.Token, not %s.' %(type(token)))
        
        for i in self.__data__['token']:
            if i._id_ == token._id_:
                raise Errors.AlreadyFound('"%s" token already found in token database.' %(token.token))
            
        self.__data__['token'].append(token)
    
    def appendTag(self, tag):
        if type(tag) != Tag:
            raise TypeError('"tag" argument must be corpse.obj.Tag, not %s.' %(type(tag)))
        
        for i in self.__data__['tag']:
            if i._id_ == tag._id_:
                raise Errors.AlreadyFound('"%s" tag already found in tag database.' %(tag.tag))
        
        self.__data__['tag'].append(tag)
    
    def removeText(self, id, tokenadress = True, tagadress = True):
        if type(id) != str:
            raise TypeError('"id" argument must be str, not %s.' %(type(id)))
        if len(self.__data__['text']) == 0:
            raise Errors.NotFound('There is no text found in database.')
        
        notfound = False
        for i in self.__data__['text']:
            if i._id_ == text._id_:
                self.__data__['text'].remove(i)
                break
            else:
                if notfound == False:
                    notfound = True
        
        if tokenadress == True:
            for i in self.__data__['token']:
                if id in i.adress:
                    i.adress.remove(id)
        
        if tagadress == True:
            for i in self.__data__['tag']:
                if id in i.adress:
                    i.adress.remove(id)
        
        if notfound == True:
            raise Errors.NotFound('Specified text not found in text database to remove.')
    
    def removeToken(self, id):
        if type(id) != str:
            raise TypeError('"id" argument must be str, not %s.' %(type(id)))
        if len(self.__data__['token']) == 0:
            raise Errors.NotFound('There is no token found in database.')
        
        notfound = False
        for i in self.__data__['token']:
            if i._id_ == id:
                self.__data__['token'].remove(i)
                break
            else:
                if notfound == False:
                    notfound = True
        
        if notfound == True:
            raise Errors.NotFound('Specified token not found in token database to remove.')
    
    def removeTag(self, id):
        if type(id) != str:
            raise TypeError('"id" argument must be str, not %s.' %(type(id)))
        if len(self.__data__['tag']) == 0:
            raise Errors.NotFound('There is no tag found in database.')
        
        notfound = False
        for i in self.__data__['tag']:
            if i._id_ == id:
                self.__data__['tag'].remove(i)
                break
            else:
                if notfound == False:
                    notfound = True
        
        if notfound == True:
            raise Errors.NotFound('Specified token not found in token database to remove.')
    
    def getText(self, id):
        if type(id) != str:
            raise TypeError('"id" argument must be str, not %s.' %(type(id)))
        if len(self.__data__['text']) == 0:
            raise Errors.NotFound('There is no text found in database.')
        
        notfound = False
        for i in self.__data__['text']:
            if i._id_ == id:
                self.__cursor__.seek(0)
                return i
            else:
                if notfound == False:
                    notfound == True
        
        if notfound == True:
            raise Errors.NotFound('Specified text not found in text database to return.')
    
    def getToken(self, id):
        if type(id) != str:
            raise TypeError('"id" argument must be str, not %s.' %(type(id)))
        if len(self.__data__['token']) == 0:
            raise Errors.NotFound('There is no token found in database.')
        
        notfound = False
        for i in self.__data__['token']:
            if i._id_ == id:
                self.__cursor__.seek(0)
                return i
            else:
                if notfound == False:
                    notfound == True
        
        if notfound == True:
            raise Errors.NotFound('Specified token not found in token database to return.')
    
    def getTag(self, id):
        if type(id) != str:
            raise TypeError('"id" argument must be str, not %s.' %(type(id)))
        if len(self.__data__['tag']) == 0:
            raise Errors.NotFound('There is no tag found in database.')
        
        notfound = False
        for i in self.__data__['tag']:
            if i._id_ == id:
                self.__cursor__.seek(0)
                return i
            else:
                if notfound == False:
                    notfound == True
        
        if notfound == True:
            raise Errors.NotFound('Specified tag not found in tag database to return.')